@@ -68,7 +68,11 @@ AccountSettings::AccountSettings(QWidget *parent) :
 
     ui->_folderList->setItemDelegate( delegate );
     ui->_folderList->setModel( _model );
+#if defined(Q_OS_MAC)
+    ui->_folderList->setMinimumWidth( 400 );
+#else
     ui->_folderList->setMinimumWidth( 300 );
+#endif
     ui->_folderList->setEditTriggers( QAbstractItemView::NoEditTriggers );
 
     ui->_buttonRemove->setEnabled(false);
@@ -102,44 +106,61 @@ AccountSettings::AccountSettings(QWidget *parent) :
 
     ui->connectLabel->setText(tr("No account configured."));
     ui->_buttonAdd->setEnabled(false);
+
+    connect(AccountManager::instance(), SIGNAL(accountChanged(Account*,Account*)),
+            this, SLOT(slotAccountChanged(Account*,Account*)));
+    slotAccountChanged(AccountManager::instance()->account(), 0);
+
+    connect(FolderMan::instance(), SIGNAL(folderListLoaded(Folder::Map)),
+            this, SLOT(setFolderList(Folder::Map)));
+    setFolderList(FolderMan::instance()->map());
+}
+
+void AccountSettings::slotAccountChanged(Account *newAccount, Account *oldAccount)
+{
+    if (oldAccount) {
+        disconnect(oldAccount, SIGNAL(stateChanged(int)), this, SLOT(slotAccountStateChanged(int)));
+        disconnect(oldAccount->quotaInfo(), SIGNAL(quotaUpdated(qint64,qint64)),
+                    this, SLOT(slotUpdateQuota(qint64,qint64)));
+    }
+
+    _account = newAccount;
     if (_account) {
         connect(_account, SIGNAL(stateChanged(int)), SLOT(slotAccountStateChanged(int)));
         slotAccountStateChanged(_account->state());
+
+        QuotaInfo *quotaInfo = _account->quotaInfo();
+        connect( quotaInfo, SIGNAL(quotaUpdated(qint64,qint64)),
+                this, SLOT(slotUpdateQuota(qint64,qint64)));
+        slotUpdateQuota(quotaInfo->lastQuotaTotalBytes(), quotaInfo->lastQuotaUsedBytes());
     }
 
-    QuotaInfo *quotaInfo = _account->quotaInfo();
-    connect( quotaInfo, SIGNAL(quotaUpdated(qint64,qint64)),
-             this, SLOT(slotUpdateQuota(qint64,qint64)));
-    slotUpdateQuota(quotaInfo->lastQuotaTotalBytes(), quotaInfo->lastQuotaUsedBytes());
-
-    setFolderList(FolderMan::instance()->map());
 }
 
 void AccountSettings::slotFolderActivated( const QModelIndex& indx )
 {
-  bool state = indx.isValid();
+  bool isValid = indx.isValid();
 
   bool haveFolders = ui->_folderList->model()->rowCount() > 0;
 
-  ui->_buttonRemove->setEnabled(state);
+  ui->_buttonRemove->setEnabled(isValid);
   if( Theme::instance()->singleSyncFolder() ) {
       // only one folder synced folder allowed.
       ui->_buttonAdd->setVisible(!haveFolders);
   } else {
       ui->_buttonAdd->setVisible(true);
-      ui->_buttonAdd->setEnabled( true );
   }
-  ui->_buttonEnable->setEnabled( state );
+  ui->_buttonAdd->setEnabled(_account && _account->state() == Account::Connected);
+  ui->_buttonEnable->setEnabled( isValid );
 
-  if ( state ) {
+  if ( isValid ) {
     bool folderEnabled = _model->data( indx, FolderStatusDelegate::FolderSyncEnabled).toBool();
-    qDebug() << "folder is sync enabled: " << folderEnabled;
     if ( folderEnabled ) {
       ui->_buttonEnable->setText( tr( "Pause" ) );
     } else {
       ui->_buttonEnable->setText( tr( "Resume" ) );
     }
-    ui->_buttonEnable->setEnabled(_account && _account->state() == Account::Connected);
+    ui->_buttonEnable->setEnabled( _account && _account->state() == Account::Connected);
   }
 }
 
@@ -151,8 +172,6 @@ void AccountSettings::slotAddFolder()
     folderMan->setSyncEnabled(false); // do not start more syncs.
 
     FolderWizard *folderWizard = new FolderWizard(this);
-    Folder::Map folderMap = folderMan->map();
-    folderWizard->setFolderMap( folderMap );
 
     connect(folderWizard, SIGNAL(accepted()), SLOT(slotFolderWizardAccepted()));
     connect(folderWizard, SIGNAL(rejected()), SLOT(slotFolderWizardRejected()));
@@ -206,11 +225,9 @@ void AccountSettings::slotAddFolder( Folder *folder )
     folderToModelItem( item, folder );
     _model->appendRow( item );
     // in order to update the enabled state of the "Sync now" button
-    connect(folder, SIGNAL(syncStateChange()), this, SLOT(slotButtonsSetEnabled()), Qt::UniqueConnection);
+    connect(folder, SIGNAL(syncStateChange()), this, SLOT(slotFolderSyncStateChange()), Qt::UniqueConnection);
 }
 
-
-
 void AccountSettings::slotButtonsSetEnabled()
 {
     QModelIndex selected = ui->_folderList->currentIndex();
@@ -221,6 +238,10 @@ void AccountSettings::slotButtonsSetEnabled()
 void AccountSettings::setGeneralErrors( const QStringList& errors )
 {
     _generalErrors = errors;
+    if (_account) {
+        // this will update the message
+        slotAccountStateChanged(_account->state());
+    }
 }
 
 void AccountSettings::folderToModelItem( QStandardItem *item, Folder *f )
@@ -245,6 +266,10 @@ void AccountSettings::folderToModelItem( QStandardItem *item, Folder *f )
                 // if the folder was disabled before, set the sync icon
                 item->setData( theme->syncStateIcon( SyncResult::SyncRunning), FolderStatusDelegate::FolderStatusIconRole );
             }  // we keep the previous icon for the SyncPrepare state.
+        } else if( status == SyncResult::Undefined ) {
+            // startup, the sync was never done.
+            qDebug() << "XXX FIRST time sync, setting icon to sync running!";
+            item->setData( theme->syncStateIcon( SyncResult::SyncRunning), FolderStatusDelegate::FolderStatusIconRole );
         } else {
             // kepp the previous icon for the prepare phase.
             if( status == SyncResult::Problem) {
@@ -337,7 +362,7 @@ void AccountSettings::slotResetCurrentFolder()
         if( ret == QMessageBox::Yes ) {
             FolderMan *folderMan = FolderMan::instance();
             Folder *f = folderMan->folder(alias);
-            f->slotTerminateSync(true);
+            f->slotTerminateSync();
             f->wipe();
             folderMan->slotScheduleAllFolders();
         }
@@ -360,6 +385,7 @@ void AccountSettings::showConnectionLabel( const QString& message, const QString
     if( _generalErrors.isEmpty() ) {
         ui->connectLabel->setText( message );
         ui->connectLabel->setToolTip(tooltip);
+        ui->connectLabel->setStyleSheet(QString());
     } else {
         const QString msg = _generalErrors.join(QLatin1String("\n"));
         ui->connectLabel->setText( msg );
@@ -390,28 +416,6 @@ void AccountSettings::setFolderList( const Folder::Map &folders )
 
 }
 
-// move from Application
-void AccountSettings::slotFolderOpenAction( const QString& alias )
-{
-    Folder *f = FolderMan::instance()->folder(alias);
-    qDebug() << "opening local url " << f->path();
-    if( f ) {
-        QUrl url(f->path(), QUrl::TolerantMode);
-        url.setScheme( QLatin1String("file") );
-
-#ifdef Q_OS_WIN
-        // work around a bug in QDesktopServices on Win32, see i-net
-        QString filePath = f->path();
-
-        if (filePath.startsWith(QLatin1String("\\\\")) || filePath.startsWith(QLatin1String("//")))
-            url.setUrl(QDir::toNativeSeparators(filePath));
-        else
-            url = QUrl::fromLocalFile(filePath);
-#endif
-        QDesktopServices::openUrl(url);
-    }
-}
-
 void AccountSettings::slotEnableCurrentFolder()
 {
     QModelIndex selected = ui->_folderList->selectionModel()->currentIndex();
@@ -428,12 +432,25 @@ void AccountSettings::slotEnableCurrentFolder()
 
             // this sets the folder status to disabled but does not interrupt it.
             Folder *f = folderMan->folder( alias );
-            if( f && folderEnabled ) {
+            if (!f) {
+                return;
+            }
+
+            if( folderEnabled ) {
                 // check if a sync is still running and if so, ask if we should terminate.
                 if( f->isBusy() ) { // its still running
-                    int reply = QMessageBox::question( 0, tr("Sync Running"),
-                                                       tr("The syncing operation is running.<br/>Do you want to terminate it?"),
-                                                       QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes );
+#if defined(Q_OS_MAC)
+                    QWidget *parent = this;
+                    Qt::WindowFlags flags = Qt::Sheet;
+#else
+                    QWidget *parent = 0;
+                    Qt::WindowFlags flags = Qt::Dialog | Qt::MSWindowsFixedSizeDialogHint; // default flags
+#endif
+                    QMessageBox msgbox(QMessageBox::Question, tr("Sync Running"),
+                                       tr("The syncing operation is running.<br/>Do you want to terminate it?"),
+                                       QMessageBox::Yes | QMessageBox::No, parent, flags);
+                    msgbox.setDefaultButton(QMessageBox::Yes);
+                    int reply = msgbox.exec();
                     if ( reply == QMessageBox::Yes )
                         terminate = true;
                     else
@@ -444,7 +461,7 @@ void AccountSettings::slotEnableCurrentFolder()
             // message box can return at any time while the thread keeps running,
             // so better check again after the user has responded.
             if ( f->isBusy() && terminate ) {
-                f->slotTerminateSync(false);
+                f->slotTerminateSync();
             }
 
             folderMan->slotEnableFolder( alias, !folderEnabled );
@@ -542,134 +559,66 @@ QString AccountSettings::shortenFilename( const QString& folder, const QString&
     return shortFile;
 }
 
-void AccountSettings::slotProgressProblem(const QString& folder, const Progress::SyncProblem& problem)
+void AccountSettings::slotSetProgress(const QString& folder, const Progress::Info &progress )
 {
-    Q_UNUSED(problem);
-
     QStandardItem *item = itemForFolder( folder );
     if( !item ) return;
 
-    int warnCount = qvariant_cast<int>( item->data(FolderStatusDelegate::WarningCount) );
-    warnCount++;
-    item->setData( QVariant(warnCount), FolderStatusDelegate::WarningCount );
-}
-
-void AccountSettings::slotSetProgress(const QString& folder, const Progress::Info &progress )
-{
-    // qDebug() << "================================> Progress for folder " << folder << " progress " << Progress::asResultString(progress.kind);
-    QStandardItem *item = itemForFolder( folder );
-    qint64 prog1 = progress.current_file_bytes;
-    qint64 prog2 = progress.file_size;
-
-    if( item == NULL ) {
-        return;
+    if(!progress._lastCompletedItem.isEmpty()
+            && Progress::isWarningKind(progress._lastCompletedItem._status)) {
+        int warnCount = item->data(FolderStatusDelegate::WarningCount).toInt();
+        warnCount++;
+        item->setData( QVariant(warnCount), FolderStatusDelegate::WarningCount );
     }
 
-    // Hotfix for a crash that I experienced in a very rare case/setup
-    if (progress.kind == Mirall::Progress::Invalid) {
-        qDebug() << "================================> INVALID Progress for folder " << folder;
-        return;
-    }
-    if( (progress.kind == Progress::StartSync)
-            && progress.overall_file_count == 0 ) {
-        // do not show progress if nothing is transmitted.
-        return;
-    }
-
-    QString itemFileName = shortenFilename(folder, progress.current_file);
-    QString syncFileProgressString;
-
-    // stay with the previous kind-string for Context.
-    if( progress.kind != Progress::Context ) {
-        _kindContext = Progress::asActionString(progress.kind);
-    } else {
-        if( _kindContext.isEmpty() ) {
-            // empty kind context means that the dialog was opened after the action
-            // was started.
-            Progress::Kind kind = ProgressDispatcher::instance()->currentFolderContext(progress.folder);
-            if( kind != Progress::Invalid ) {
-                _kindContext = Progress::asActionString(kind);
-            }
+    // find the single item to display:  This is going to be the bigger item, or the last completed
+    // item if no items are in progress.
+    SyncFileItem curItem = progress._lastCompletedItem;
+    qint64 curItemProgress = -1; // -1 means finished
+    quint64 biggerItemSize = -1;
+    foreach(const Progress::Info::ProgressItem &citm, progress._currentItems) {
+        if (curItemProgress == -1 || (Progress::isSizeDependent(citm._item._instruction)
+                                      && biggerItemSize < citm._item._size)) {
+            curItemProgress = citm._completedSize;
+            curItem = citm._item;
+            biggerItemSize = citm._item._size;
         }
     }
-    QString kindString = _kindContext;
-
-    switch( progress.kind ) {
-    case Progress::StartSync:
-        item->setData( QVariant(0), FolderStatusDelegate::WarningCount );
-        break;
-    case Progress::StartDownload:
-    case Progress::StartUpload:
-    case Progress::StartDelete:
-    case Progress::StartRename:
-        syncFileProgressString = tr("Start");
-        if( _hideProgressTimers.contains(item) ) {
-            // The timer is still running.
-            QTimer *t = _hideProgressTimers.take(item);
-            t->stop();
-            t->deleteLater();
-        }
-        break;
-    case Progress::Context:
-        syncFileProgressString = tr("Currently");
-        break;
-    case Progress::EndDownload:
-    case Progress::EndUpload:
-    case Progress::EndDelete:
-    case Progress::EndRename:
-        break;
-    case Progress::EndSync:
-        syncFileProgressString = tr("Completely");
-
-        // start a timer to stop the progress display
-        QTimer *timer;
-        if( _hideProgressTimers.contains(item) ) {
-            timer = _hideProgressTimers[item];
-            // there is already one timer running.
-        } else {
-            timer = new QTimer;
-            connect(timer, SIGNAL(timeout()), this, SLOT(slotHideProgress()));
-            timer->setSingleShot(true);
-            _hideProgressTimers.insert(item, timer);
-        }
-        timer->start(5000);
-        break;
-    case Progress::Invalid:
-    case Progress::Download:
-    case Progress::Upload:
-    case Progress::Inactive:
-    case Progress::SoftError:
-    case Progress::NormalError:
-    case Progress::FatalError:
-        break;
+    if (curItemProgress == -1) {
+        curItemProgress = curItem._size;
     }
 
-    QString fileProgressString;
-    QString s1 = Utility::octetsToString( prog1 );
-    QString s2 = Utility::octetsToString( prog2 );
+    QString itemFileName = shortenFilename(folder, curItem._file);
+    QString kindString = Progress::asActionString(curItem);
 
     // switch on extra space.
     item->setData( QVariant(true), FolderStatusDelegate::AddProgressSpace );
 
-    if( progress.kind != Progress::EndSync ) {
-        // Example text: "Currently uploading foobar.png (1MB of 2MB)"
-        fileProgressString = tr("%1 %2 %3 (%4 of %5)").arg(syncFileProgressString).arg(kindString).
-                arg(itemFileName).arg(s1).arg(s2);
+    QString fileProgressString;
+    if (Progress::isSizeDependent(curItem._instruction)) {
+        QString s1 = Utility::octetsToString( curItemProgress );
+        QString s2 = Utility::octetsToString( curItem._size );
+        //: Example text: "uploading foobar.png (1MB of 2MB)"
+        fileProgressString = tr("%1 %2 (%3 of %4)").arg(kindString, itemFileName, s1, s2);
     } else {
-        fileProgressString = tr("Completely finished.");
+        //: Example text: "uploading foobar.png"
+        fileProgressString = tr("%1 %2").arg(kindString, itemFileName);
     }
     item->setData( fileProgressString,FolderStatusDelegate::SyncProgressItemString);
 
     // overall progress
-    s1 = Utility::octetsToString( progress.overall_current_bytes );
-    s2 = Utility::octetsToString( progress.overall_transmission_size );
-    QString overallSyncString = tr("%1 of %2, file %3 of %4").arg(s1).arg(s2)
-            .arg(progress.current_file_no).arg(progress.overall_file_count);
+    quint64 completedSize = progress.completedSize();
+    quint64 currentFile =  progress._completedFileCount + progress._currentItems.count();
+    QString s1 = Utility::octetsToString( completedSize );
+    QString s2 = Utility::octetsToString( progress._totalSize );
+    QString overallSyncString = tr("%1 of %2, file %3 of %4").arg(s1, s2)
+            .arg(currentFile).arg(progress._totalFileCount);
     item->setData( overallSyncString, FolderStatusDelegate::SyncProgressOverallString );
 
     int overallPercent = 0;
-    if( progress.overall_transmission_size > 0 ) {
-        overallPercent = qRound(double(progress.overall_current_bytes)/double(progress.overall_transmission_size) * 100.0);
+    if( progress._totalFileCount > 0 ) {
+        // Add one 'byte' for each files so the percentage is moving when deleting or renaming files
+        overallPercent = qRound(double(completedSize + progress._completedFileCount)/double(progress._totalSize + progress._totalFileCount) * 100.0);
     }
     item->setData( overallPercent, FolderStatusDelegate::SyncProgressOverallPercent);
 }
@@ -691,10 +640,10 @@ void AccountSettings::slotHideProgress()
             }
 
             if( ok ) {
-                item->setData( QVariant(false),  FolderStatusDelegate::AddProgressSpace );
-                item->setData( QVariant(QString::null), FolderStatusDelegate::SyncProgressOverallString );
-                item->setData( QVariant(QString::null), FolderStatusDelegate::SyncProgressItemString );
-                item->setData( 0,                       FolderStatusDelegate::SyncProgressOverallPercent );
+                item->setData( false,     FolderStatusDelegate::AddProgressSpace );
+                item->setData( QString(), FolderStatusDelegate::SyncProgressOverallString );
+                item->setData( QString(), FolderStatusDelegate::SyncProgressItemString );
+                item->setData( 0,         FolderStatusDelegate::SyncProgressOverallPercent );
             }
             _hideProgressTimers.remove(item);
             break;
@@ -705,6 +654,35 @@ void AccountSettings::slotHideProgress()
     send_timer->deleteLater();
 }
 
+void AccountSettings::slotFolderSyncStateChange()
+{
+    slotButtonsSetEnabled();
+    Folder* folder = qobject_cast<Folder *>(sender());
+    if (!folder) return;
+
+    QStandardItem *item = itemForFolder( folder->alias() );
+    if( !item ) return;
+
+    SyncResult::Status state = folder->syncResult().status();
+    if (state == SyncResult::SyncPrepare)  {
+        item->setData( QVariant(0), FolderStatusDelegate::WarningCount );
+    } else if (state == SyncResult::Success || state == SyncResult::Problem) {
+        // start a timer to stop the progress display
+        QTimer *timer;
+        if( _hideProgressTimers.contains(item) ) {
+            timer = _hideProgressTimers[item];
+            // there is already one timer running.
+        } else {
+            timer = new QTimer(this);
+            connect(timer, SIGNAL(timeout()), this, SLOT(slotHideProgress()));
+            timer->setSingleShot(true);
+            _hideProgressTimers.insert(item, timer);
+        }
+        timer->start(5000);
+    }
+}
+
+
 void AccountSettings::slotUpdateQuota(qint64 total, qint64 used)
 {
     if( total > 0 ) {
@@ -720,7 +698,7 @@ void AccountSettings::slotUpdateQuota(qint64 total, qint64 used)
         QString totalStr = Utility::octetsToString(total);
         double percent = used/(double)total*100;
         QString percentStr = Utility::compactFormatDouble(percent, 1);
-        ui->quotaLabel->setText(tr("%1 of %2 (%3%) in use.").arg(usedStr, totalStr, percentStr));
+        ui->quotaLabel->setText(tr("%1 (%3%) of %2 server space in use.").arg(usedStr, totalStr, percentStr));
     } else {
         ui->quotaProgressBar->setVisible(false);
         ui->quotaInfoLabel->setVisible(false);
@@ -745,7 +723,7 @@ void AccountSettings::slotAccountStateChanged(int state)
         ui->sslButton->updateAccountInfo(_account);
         QUrl safeUrl(_account->url());
         safeUrl.setPassword(QString()); // Remove the password from the URL to avoid showing it in the UI
-        ui->_buttonAdd->setEnabled(state == Account::Connected);
+        slotButtonsSetEnabled();
         if (state == Account::Connected) {
             QString user;
             if (AbstractCredentials *cred = _account->credentials()) {
