@@ -489,7 +489,7 @@ void CSyncThread::startSync()
 
     // csync_set_auth_callback( _csync_ctx, getauth );
     csync_set_log_callback( csyncLogCatcher );
-    csync_set_log_level( 11 );
+    csync_set_log_level( 11 ); // rd: set this lower to reduce owncloudcmd output
 
     _syncTime.start();
 
