@@ -25,12 +25,11 @@
 #include "mirall/syncresult.h"
 #include "mirall/utility.h"
 #include "mirall/clientproxy.h"
+#include "mirall/syncengine.h"
+#include "mirall/syncrunfilelog.h"
 
 #include "creds/abstractcredentials.h"
 
-#include <QThread>
-
-
 extern "C" {
 
 enum csync_exclude_type_e {
@@ -49,10 +48,12 @@ CSYNC_EXCLUDE_TYPE csync_excluded(CSYNC *ctx, const char *path, int filetype);
 #include <QDebug>
 #include <QTimer>
 #include <QUrl>
-#include <QFileSystemWatcher>
 #include <QDir>
+
+#ifndef TOKEN_AUTH_ONLY
 #include <QMessageBox>
 #include <QPushButton>
+#endif
 
 namespace Mirall {
 
@@ -62,8 +63,6 @@ Folder::Folder(const QString &alias, const QString &path, const QString& secondP
       , _remotePath(secondPath)
       , _alias(alias)
       , _enabled(true)
-      , _thread(0)
-      , _csync(0)
       , _csyncError(false)
       , _csyncUnavail(false)
       , _wipeDb(false)
@@ -76,9 +75,6 @@ Folder::Folder(const QString &alias, const QString &path, const QString& secondP
 
     MirallConfigFile cfg;
 
-    // QObject::connect(_watcher, SIGNAL(folderChanged(const QStringList &)),
-    //                 SLOT(slotChanged(const QStringList &)));
-
     _syncResult.setStatus( SyncResult::NotYetStarted );
 
     // check if the local path exists
@@ -95,12 +91,31 @@ Folder::Folder(const QString &alias, const QString &path, const QString& secondP
 
 bool Folder::init()
 {
-    QString url = Utility::toCSyncScheme(remoteUrl().toString());
+    Account *account = AccountManager::instance()->account();
+    if (!account) {
+        // Normaly this should not happen, but it could be that there is something
+        // wrong with the config and it is better not to crash.
+        qWarning() << "WRN: No account  configured, can't sync";
+        return false;
+    }
+
+    // We need to reconstruct the url because the path need to be fully decoded, as csync will  re-encode the path:
+    //  Remember that csync will just append the filename to the path and pass it to the vio plugin.
+    //  csync_owncloud will then re-encode everything.
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    QUrl url = remoteUrl();
+    QString url_string = url.scheme() + QLatin1String("://") + url.authority(QUrl::EncodeDelimiters) + url.path(QUrl::FullyDecoded);
+#else
+    // Qt4 was broken anyway as it did not encode the '#' as it should have done  (it was actually a provlem when parsing the path from QUrl::setPath
+    QString url_string = remoteUrl().toString();
+#endif
+    url_string = Utility::toCSyncScheme(url_string);
+
     QString localpath = path();
 
-    if( csync_create( &_csync_ctx, localpath.toUtf8().data(), url.toUtf8().data() ) < 0 ) {
+    if( csync_create( &_csync_ctx, localpath.toUtf8().data(), url_string.toUtf8().data() ) < 0 ) {
         qDebug() << "Unable to create csync-context!";
-        slotCSyncError(tr("Unable to create csync-context"));
+        slotSyncError(tr("Unable to create csync-context"));
         _csync_ctx = 0;
     } else {
         csync_set_log_callback( csyncLogCatcher );
@@ -109,7 +124,6 @@ bool Folder::init()
         MirallConfigFile cfgFile;
         csync_set_config_dir( _csync_ctx, cfgFile.configPath().toUtf8() );
 
-        csync_enable_conflictcopys(_csync_ctx);
         setIgnoredFiles();
         if (Account *account = AccountManager::instance()->account()) {
             account->credentials()->syncContextPreInit(_csync_ctx);
@@ -119,13 +133,13 @@ bool Folder::init()
 
         if( csync_init( _csync_ctx ) < 0 ) {
             qDebug() << "Could not initialize csync!" << csync_get_status(_csync_ctx) << csync_get_status_string(_csync_ctx);
-            QString errStr = CSyncThread::csyncErrorToString(CSYNC_STATUS(csync_get_status(_csync_ctx)));
+            QString errStr = SyncEngine::csyncErrorToString(CSYNC_STATUS(csync_get_status(_csync_ctx)));
             const char *errMsg = csync_get_status_string(_csync_ctx);
             if( errMsg ) {
                 errStr += QLatin1String("<br/>");
                 errStr += QString::fromUtf8(errMsg);
             }
-            slotCSyncError(errStr);
+            slotSyncError(errStr);
             csync_destroy(_csync_ctx);
             _csync_ctx = 0;
         }
@@ -135,11 +149,10 @@ bool Folder::init()
 
 Folder::~Folder()
 {
-    if( _thread ) {
-        _csync->abort();
-        _thread->wait();
+    if( _engine ) {
+        _engine->abort();
+        _engine.reset(0);
     }
-    delete _csync;
     // Destroy csync here.
     csync_destroy(_csync_ctx);
 }
@@ -170,14 +183,6 @@ void Folder::checkLocalPath()
             _syncResult.setStatus( SyncResult::SetupError );
         }
     }
-
-    // if all is fine, connect a FileSystemWatcher
-    if( _syncResult.status() != SyncResult::SetupError ) {
-        _pathWatcher = new QFileSystemWatcher(this);
-        _pathWatcher->addPath( _path );
-        connect(_pathWatcher, SIGNAL(directoryChanged(QString)),
-                SLOT(slotLocalPathChanged(QString)));
-    }
 }
 
 QString Folder::alias() const
@@ -196,7 +201,7 @@ QString Folder::path() const
 
 bool Folder::isBusy() const
 {
-    return ( _thread && _thread->isRunning() );
+    return !_engine.isNull();
 }
 
 QString Folder::remotePath() const
@@ -214,7 +219,6 @@ QUrl Folder::remoteUrl() const
     }
     path.append(_remotePath);
     url.setPath(path);
-    qDebug() << url;
     return url;
 }
 
@@ -314,23 +318,31 @@ void Folder::bubbleUpSyncResult()
     SyncFileItem firstItemRenamed;
     Logger *logger = Logger::instance();
 
-    foreach (const SyncFileItem &item, _syncResult.syncFileItemVector() ) {
-        if( item._status == SyncFileItem::FatalError || item._status == SyncFileItem::NormalError ) {
-            slotCSyncError( tr("File %1: %2").arg(item._file).arg(item._errorString) );
-            logger->postOptionalGuiLog(tr("File %1").arg(item._file), item._errorString);
+    SyncRunFileLog syncFileLog;
 
+    syncFileLog.start(path(), _engine ? _engine->stopWatch() : Utility::StopWatch() );
+
+    QElapsedTimer timer;
+    timer.start();
+
+    foreach (const SyncFileItem &item, _syncResult.syncFileItemVector() ) {
+        // Log the item
+        syncFileLog.logItem( item );
+
+        // and process the item to the gui
+        if( item._status == SyncFileItem::FatalError || item._status == SyncFileItem::NormalError ) {
+            slotSyncError( tr("%1: %2").arg(item._file, item._errorString) );
+            logger->postOptionalGuiLog(item._file, item._errorString);
         } else {
             // add new directories or remove gone away dirs to the watcher
-            if (item._type == SyncFileItem::Directory &&
-                    item._instruction == CSYNC_INSTRUCTION_NEW ) {
+            if (item._isDirectory && item._instruction == CSYNC_INSTRUCTION_NEW ) {
                 FolderMan::instance()->addMonitorPath( alias(), path()+item._file );
             }
-            if (item._type == SyncFileItem::Directory &&
-                    item._instruction == CSYNC_INSTRUCTION_REMOVE ) {
+            if (item._isDirectory && item._instruction == CSYNC_INSTRUCTION_REMOVE ) {
                 FolderMan::instance()->removeMonitorPath( alias(), path()+item._file );
             }
 
-            if (item._dir == SyncFileItem::Down) {
+            if (item._direction == SyncFileItem::Down) {
                 switch (item._instruction) {
                 case CSYNC_INSTRUCTION_NEW:
                     newItems++;
@@ -361,46 +373,89 @@ void Folder::bubbleUpSyncResult()
                     // nothing.
                     break;
                 }
-            } else if( item._dir == SyncFileItem::None ) { // ignored files counting.
+            } else if( item._direction == SyncFileItem::None ) { // ignored files counting.
                 if( item._instruction == CSYNC_INSTRUCTION_IGNORE ) {
                     ignoredItems++;
                 }
             }
         }
     }
+    syncFileLog.close();
 
+    qDebug() << "Processing result list and logging took " << timer.elapsed() << " Milliseconds.";
     _syncResult.setWarnCount(ignoredItems);
 
-
-    createGuiLog( firstItemNew._file,     tr("downloaded"), newItems );
-    createGuiLog( firstItemDeleted._file, tr("removed"), removedItems );
-    createGuiLog( firstItemUpdated._file, tr("updated"), updatedItems );
+    createGuiLog( firstItemNew._file,     FILE_STATUS_NEW, newItems );
+    createGuiLog( firstItemDeleted._file, FILE_STATUS_REMOVE, removedItems );
+    createGuiLog( firstItemUpdated._file, FILE_STATUS_UPDATED, updatedItems );
 
     if( !firstItemRenamed.isEmpty() ) {
-        QString renameVerb = tr("renamed");
+        SyncFileStatus status = FILE_STATUS_RENAME;
         // if the path changes it's rather a move
         QDir renTarget = QFileInfo(firstItemRenamed._renameTarget).dir();
         QDir renSource = QFileInfo(firstItemRenamed._file).dir();
         if(renTarget != renSource) {
-            renameVerb = tr("moved");
+            status = FILE_STATUS_MOVE;
         }
-        createGuiLog( firstItemRenamed._file, tr("%1 to %2").arg(renameVerb).arg(firstItemRenamed._renameTarget), renamedItems );
+        createGuiLog( firstItemRenamed._file, status, renamedItems, firstItemRenamed._renameTarget );
     }
 
     qDebug() << "OO folder slotSyncFinished: result: " << int(_syncResult.status());
 }
 
-void Folder::createGuiLog( const QString& filename, const QString& verb, int count )
+void Folder::createGuiLog( const QString& filename, SyncFileStatus status, int count,
+                           const QString& renameTarget )
 {
     if(count > 0) {
         Logger *logger = Logger::instance();
 
         QString file = QDir::toNativeSeparators(filename);
-        if (count == 1) {
-            logger->postOptionalGuiLog(tr("File %1").arg(verb), tr("'%1' has been %2.").arg(file).arg(verb));
-        } else {
-            logger->postOptionalGuiLog(tr("Files %1").arg(verb),
-                                       tr("'%1' and %2 other files have been %3.").arg(file).arg(count-1).arg(verb));
+        QString text;
+
+        // not all possible values of status are evaluated here because the others
+        // are not used in the calling function. Please check there.
+        switch (status) {
+        case FILE_STATUS_REMOVE:
+            if( count > 1 ) {
+                text = tr("%1 and %2 other files have been removed.", "%1 names a file.").arg(file).arg(count-1);
+            } else {
+                text = tr("%1 has been removed.", "%1 names a file.").arg(file);
+            }
+            break;
+        case FILE_STATUS_NEW:
+            if( count > 1 ) {
+                text = tr("%1 and %2 other files have been downloaded.", "%1 names a file.").arg(file).arg(count-1);
+            } else {
+                text = tr("%1 has been downloaded.", "%1 names a file.").arg(file);
+            }
+            break;
+        case FILE_STATUS_UPDATED:
+            if( count > 1 ) {
+                text = tr("%1 and %2 other files have been updated.").arg(file).arg(count-1);
+            } else {
+                text = tr("%1 has been updated.", "%1 names a file.").arg(file);
+            }
+            break;
+        case FILE_STATUS_RENAME:
+            if( count > 1 ) {
+                text = tr("%1 has been renamed to %2 and %3 other files have been renamed.").arg(file).arg(renameTarget).arg(count-1);
+            } else {
+                text = tr("%1 has been renamed to %2.", "%1 and %2 name files.").arg(file).arg(renameTarget);
+            }
+            break;
+        case FILE_STATUS_MOVE:
+            if( count > 1 ) {
+                text = tr("%1 has been moved to %2 and %3 other files have been moved.").arg(file).arg(renameTarget).arg(count-1);
+            } else {
+                text = tr("%1 has been moved to %2.").arg(file).arg(renameTarget);
+            }
+            break;
+        default:
+            break;
+        }
+
+        if( !text.isEmpty() ) {
+            logger->postOptionalGuiLog( tr("Sync Activity"), text );
         }
     }
 }
@@ -415,26 +470,6 @@ int Folder::slotWipeBlacklist()
     return _journal.wipeBlacklist();
 }
 
-void Folder::slotLocalPathChanged( const QString& dir )
-{
-    QDir notifiedDir(dir);
-    QDir localPath( path() );
-
-    if( notifiedDir.absolutePath() == localPath.absolutePath() ) {
-        if( !localPath.exists() ) {
-            qDebug() << "XXXXXXX The sync folder root was removed!!";
-            if( _thread && _thread->isRunning() ) {
-                qDebug() << "CSync currently running, set wipe flag!!";
-            } else {
-                qDebug() << "CSync not running, wipe it now!!";
-                wipe();
-            }
-
-            qDebug() << "ALARM: The local path was DELETED!";
-        }
-    }
-}
-
 void Folder::setConfigFile( const QString& file )
 {
     _configFile = file;
@@ -450,34 +485,25 @@ void Folder::slotThreadTreeWalkResult(const SyncFileItemVector& items)
     _syncResult.setSyncFileItemVector(items);
 }
 
-void Folder::slotTerminateSync(bool block)
+void Folder::slotTerminateSync()
 {
     qDebug() << "folder " << alias() << " Terminating!";
 
-    if( _thread && _csync ) {
-        _csync->abort();
+    if( _engine ) {
+        _engine->abort();
 
         // Do not display an error message, user knows his own actions.
         // _errors.append( tr("The CSync thread terminated.") );
         // _csyncError = true;
-        if (!block) {
-            setSyncState(SyncResult::SyncAbortRequested);
-            return;
-        }
-
-        _thread->wait();
-        _csync->deleteLater();
-        delete _thread;
-        _thread = 0;
-        slotCSyncFinished();
+        setSyncEnabled(false);
+        setSyncState(SyncResult::SyncAbortRequested);
+        return;
     }
-    setSyncEnabled(false);
 }
 
-// This removes the csync File database if the sync folder definition is removed
-// permanentely. This is needed to provide a clean startup again in case another
+// This removes the csync File database
+// This is needed to provide a clean startup again in case another
 // local folder is synced to the same ownCloud.
-// See http://bugs.owncloud.org/thebuggenie/owncloud/issues/oc-788
 void Folder::wipe()
 {
     QString stateDbFile = path()+QLatin1String(".csync_journal.db");
@@ -547,14 +573,10 @@ void Folder::startSync(const QStringList &pathList)
         setProxyDirty(false);
     }
 
-    if (_thread && _thread->isRunning()) {
+    if (isBusy()) {
         qCritical() << "* ERROR csync is still running and new sync requested.";
         return;
     }
-    if (_thread)
-        _thread->quit();
-    delete _csync;
-    delete _thread;
     _errors.clear();
     _csyncError = false;
     _csyncUnavail = false;
@@ -565,32 +587,26 @@ void Folder::startSync(const QStringList &pathList)
 
 
     qDebug() << "*** Start syncing";
-    _thread = new QThread(this);
     setIgnoredFiles();
-    _csync = new CSyncThread( _csync_ctx, path(), remoteUrl().path(), &_journal);
-    _csync->moveToThread(_thread);
+    _engine.reset(new SyncEngine( _csync_ctx, path(), remoteUrl().path(), _remotePath, &_journal));
 
     qRegisterMetaType<SyncFileItemVector>("SyncFileItemVector");
     qRegisterMetaType<SyncFileItem::Direction>("SyncFileItem::Direction");
 
-    connect( _csync, SIGNAL(treeWalkResult(const SyncFileItemVector&)),
+    connect( _engine.data(), SIGNAL(treeWalkResult(const SyncFileItemVector&)),
               this, SLOT(slotThreadTreeWalkResult(const SyncFileItemVector&)), Qt::QueuedConnection);
 
-    connect(_csync, SIGNAL(started()),  SLOT(slotCSyncStarted()), Qt::QueuedConnection);
-    connect(_csync, SIGNAL(finished()), SLOT(slotCSyncFinished()), Qt::QueuedConnection);
-    connect(_csync, SIGNAL(csyncError(QString)), SLOT(slotCSyncError(QString)), Qt::QueuedConnection);
-    connect(_csync, SIGNAL(csyncUnavailable()), SLOT(slotCsyncUnavailable()), Qt::QueuedConnection);
+    connect(_engine.data(), SIGNAL(started()),  SLOT(slotSyncStarted()), Qt::QueuedConnection);
+    connect(_engine.data(), SIGNAL(finished()), SLOT(slotSyncFinished()), Qt::QueuedConnection);
+    connect(_engine.data(), SIGNAL(csyncError(QString)), SLOT(slotSyncError(QString)), Qt::QueuedConnection);
+    connect(_engine.data(), SIGNAL(csyncUnavailable()), SLOT(slotCsyncUnavailable()), Qt::QueuedConnection);
 
-    //blocking connection so the message box happens in this thread, but block the csync thread.
-    connect(_csync, SIGNAL(aboutToRemoveAllFiles(SyncFileItem::Direction,bool*)),
-                    SLOT(slotAboutToRemoveAllFiles(SyncFileItem::Direction,bool*)), Qt::BlockingQueuedConnection);
-    connect(_csync, SIGNAL(transmissionProgress(Progress::Info)), this, SLOT(slotTransmissionProgress(Progress::Info)));
-    connect(_csync, SIGNAL(transmissionProblem(Progress::SyncProblem)), this, SLOT(slotTransmissionProblem(Progress::SyncProblem)));
+    //direct connection so the message box is blocking the sync.
+    connect(_engine.data(), SIGNAL(aboutToRemoveAllFiles(SyncFileItem::Direction,bool*)),
+                    SLOT(slotAboutToRemoveAllFiles(SyncFileItem::Direction,bool*)));
+    connect(_engine.data(), SIGNAL(transmissionProgress(Progress::Info)), this, SLOT(slotTransmissionProgress(Progress::Info)));
 
-    _thread->start();
-    _thread->setPriority(QThread::LowPriority);
-
-    QMetaObject::invokeMethod(_csync, "startSync", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(_engine.data(), "startSync", Qt::QueuedConnection);
 
     // disable events until syncing is done
     // _watcher->setEventsEnabled(false);
@@ -600,18 +616,18 @@ void Folder::startSync(const QStringList &pathList)
 
 void Folder::setDirtyNetworkLimits()
 {
-    if( _csync ) {
-        QMetaObject::invokeMethod(_csync, "setNetworkLimits", Qt::QueuedConnection);
+    if (_engine) {
+        _engine->setNetworkLimits();
     }
 }
 
-void Folder::slotCSyncError(const QString& err)
+void Folder::slotSyncError(const QString& err)
 {
     _errors.append( err );
     _csyncError = true;
 }
 
-void Folder::slotCSyncStarted()
+void Folder::slotSyncStarted()
 {
     qDebug() << "    * csync thread started";
     _syncResult.setStatus(SyncResult::SyncRunning);
@@ -623,14 +639,17 @@ void Folder::slotCsyncUnavailable()
     _csyncUnavail = true;
 }
 
-void Folder::slotCSyncFinished()
+void Folder::slotSyncFinished()
 {
     qDebug() << "-> CSync Finished slot with error " << _csyncError << "warn count" << _syncResult.warnCount();
+
+    bubbleUpSyncResult();
+
+    _engine.reset(0);
     // _watcher->setEventsEnabledDelayed(2000);
     _pollTimer.start();
     _timeSinceLastSync.restart();
 
-    bubbleUpSyncResult();
 
     if (_csyncError) {
         _syncResult.setStatus(SyncResult::Error);
@@ -646,65 +665,45 @@ void Folder::slotCSyncFinished()
         _syncResult.setStatus(SyncResult::Success);
     }
 
-    if( _thread && _thread->isRunning() ) {
-        _thread->quit();
-    }
     emit syncStateChange();
+
+    // The syncFinished result that is to be triggered here makes the folderman
+    // clearing the current running sync folder marker.
+    // Lets wait a bit to do that because, as long as this marker is not cleared,
+    // file system change notifications are ignored for that folder. And it takes
+    // some time under certain conditions to make the file system notifications
+    // all come in.
+    QTimer::singleShot(200, this, SLOT(slotEmitFinishedDelayed() ));
+
+}
+
+void Folder::slotEmitFinishedDelayed()
+{
     emit syncFinished( _syncResult );
 }
 
-// the problem comes without a folder and the valid path set. Add that here
-// and hand the result over to the progress dispatcher.
-void Folder::slotTransmissionProblem( const Progress::SyncProblem& problem )
-{
-    Progress::SyncProblem newProb = problem;
-    newProb.folder = alias();
 
-    if(newProb.current_file.startsWith(QLatin1String("ownclouds://")) ||
-            newProb.current_file.startsWith(QLatin1String("owncloud://")) ) {
-        // rip off the whole ownCloud URL.
-        newProb.current_file.remove(Utility::toCSyncScheme(remoteUrl().toString()));
-    }
-    QString localPath = path();
-    if( newProb.current_file.startsWith(localPath) ) {
-        // remove the local dir.
-        newProb.current_file = newProb.current_file.right( newProb.current_file.length() - localPath.length());
-    }
-
-    // Count all error conditions.
-    _syncResult.setWarnCount( _syncResult.warnCount()+1 );
-
-    ProgressDispatcher::instance()->setProgressProblem(alias(), newProb);
-}
 
 // the progress comes without a folder and the valid path set. Add that here
 // and hand the result over to the progress dispatcher.
-void Folder::slotTransmissionProgress(const Progress::Info& progress)
+void Folder::slotTransmissionProgress(const Progress::Info &pi)
 {
-    Progress::Info newInfo = progress;
-    newInfo.folder = alias();
-
-    if(newInfo.current_file.startsWith(QLatin1String("ownclouds://")) ||
-            newInfo.current_file.startsWith(QLatin1String("owncloud://")) ) {
-        // rip off the whole ownCloud URL.
-        newInfo.current_file.remove(Utility::toCSyncScheme(remoteUrl().toString()));
-    }
-    QString localPath = path();
-    if( newInfo.current_file.startsWith(localPath) ) {
-        // remove the local dir.
-        newInfo.current_file = newInfo.current_file.right( newInfo.current_file.length() - localPath.length());
+    if (!pi._lastCompletedItem.isEmpty()
+            && Progress::isWarningKind(pi._lastCompletedItem._status)) {
+        // Count all error conditions.
+        _syncResult.setWarnCount(_syncResult.warnCount()+1);
     }
 
     // remember problems happening to set the correct Sync status in slot slotCSyncFinished.
-    if( newInfo.kind == Progress::StartSync ) {
+    if( pi._completedFileCount ) {
         _syncResult.setWarnCount(0);
     }
-
-    ProgressDispatcher::instance()->setProgressInfo(alias(), newInfo);
+    ProgressDispatcher::instance()->setProgressInfo(alias(), pi);
 }
 
 void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction direction, bool *cancel)
 {
+#ifndef TOKEN_AUTH_ONLY
     QString msg = direction == SyncFileItem::Down ?
         tr("This sync would remove all the files in the local sync folder '%1'.\n"
            "If you or your administrator have reset your account on the server, choose "
@@ -724,7 +723,11 @@ void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction direction, bool *
     *cancel = msgBox.clickedButton() == keepBtn;
     if (*cancel) {
         wipe();
+        // speed up next sync
+        _lastEtag = QString();
+        QTimer::singleShot(50, this, SLOT(slotPollTimerTimeout()));
     }
+#endif
 }
 
 SyncFileStatus Folder::fileStatus( const QString& fileName )
