@@ -27,8 +27,10 @@
 #include <shlobj.h>
 #endif
 
-#include <QDesktopServices>
+#ifndef TOKEN_AUTH_ONLY
 #include <QMessageBox>
+#endif
+
 #include <QtCore>
 
 namespace Mirall {
@@ -39,13 +41,6 @@ FolderMan::FolderMan(QObject *parent) :
     QObject(parent),
     _syncEnabled( true )
 {
-    // if QDir::mkpath would not be so stupid, I would not need to have this
-    // duplication of folderConfigPath() here
-    MirallConfigFile cfg;
-    QDir storageDir(cfg.configPath());
-    storageDir.mkpath(QLatin1String("folders"));
-    _folderConfigPath = cfg.configPath() + QLatin1String("folders");
-
     _folderChangeSignalMapper = new QSignalMapper(this);
     connect(_folderChangeSignalMapper, SIGNAL(mapped(const QString &)),
             this, SIGNAL(folderSyncStateChange(const QString &)));
@@ -53,15 +48,14 @@ FolderMan::FolderMan(QObject *parent) :
     _folderWatcherSignalMapper = new QSignalMapper(this);
     connect(_folderWatcherSignalMapper, SIGNAL(mapped(const QString&)),
             this, SLOT(slotScheduleSync(const QString&)));
+
+    ne_sock_init();
+    Q_ASSERT(!_instance);
+    _instance = this;
 }
 
 FolderMan *FolderMan::instance()
 {
-    if(!_instance) {
-        _instance = new FolderMan;
-        ne_sock_init();
-    }
-
     return _instance;
 }
 
@@ -69,6 +63,7 @@ FolderMan::~FolderMan()
 {
     qDeleteAll(_folderMap);
     ne_sock_exit();
+    _instance = 0;
 }
 
 Mirall::Folder::Map FolderMan::map()
@@ -142,25 +137,34 @@ int FolderMan::setupFolders()
 
   unloadAllFolders();
 
+  MirallConfigFile cfg;
+  QDir storageDir(cfg.configPath());
+  storageDir.mkpath(QLatin1String("folders"));
+  _folderConfigPath = cfg.configPath() + QLatin1String("folders");
+
   QDir dir( _folderConfigPath );
-  dir.setFilter(QDir::Files);
+  //We need to include hidden files just in case the alias starts with '.'
+  dir.setFilter(QDir::Files | QDir::Hidden);
   QStringList list = dir.entryList();
 
   foreach ( const QString& alias, list ) {
     Folder *f = setupFolderFromConfigFile( alias );
     if( f ) {
-        registerFolderMonitor(f);
         slotScheduleSync(alias);
         emit( folderSyncStateChange( f->alias() ) );
     }
   }
+
+  emit folderListLoaded(_folderMap);
+
   // return the number of valid folders.
   return _folderMap.size();
 }
 
 bool FolderMan::ensureJournalGone(const QString &localPath)
 {
-
+	// FIXME move this to UI, not libowncloudsync
+#ifndef TOKEN_AUTH_ONLY
     // remove old .csync_journal file
     QString stateDbFile = localPath+QLatin1String("/.csync_journal.db");
     while (QFile::exists(stateDbFile) && !QFile::remove(stateDbFile)) {
@@ -174,6 +178,7 @@ bool FolderMan::ensureJournalGone(const QString &localPath)
             return false;
         }
     }
+#endif
     return true;
 }
 
@@ -294,6 +299,7 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
     qDebug() << "Adding folder to Folder Map " << folder;
     _folderMap[alias] = folder;
     if (paused) {
+        folder->setSyncEnabled(!paused);
         _disabledFolders.insert(folder);
     }
 
@@ -304,6 +310,8 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
     connect(folder, SIGNAL(syncFinished(SyncResult)), SLOT(slotFolderSyncFinished(SyncResult)));
 
     _folderChangeSignalMapper->setMapping( folder, folder->alias() );
+
+    registerFolderMonitor(folder);
     return folder;
 }
 
@@ -344,9 +352,10 @@ void FolderMan::terminateSyncProcess( const QString& alias )
     if( ! folderAlias.isEmpty() && _folderMap.contains(folderAlias) ) {
         Folder *f = _folderMap[folderAlias];
         if( f ) {
-            f->slotTerminateSync(true);
-            if(_currentSyncFolder == folderAlias )
+            f->slotTerminateSync();
+            if(_currentSyncFolder == folderAlias ) {
                 _currentSyncFolder.clear();
+            }
         }
     }
 }
@@ -393,7 +402,7 @@ void FolderMan::slotScheduleSync( const QString& alias )
     if( ! _scheduleQueue.contains(alias ) && _folderMap.contains(alias) ) {
         Folder *f = _folderMap[alias];
         if( f ) {
-            if( f->syncEnabled() ) { // FIXME: check if that is ok
+            if( f->syncEnabled() ) {
                 f->prepareToSync();
             } else {
                 qDebug() << "Folder is not enabled, not scheduled!";
@@ -401,12 +410,12 @@ void FolderMan::slotScheduleSync( const QString& alias )
             }
         }
         _scheduleQueue.enqueue(alias);
+
     } else {
         qDebug() << " II> Sync for folder " << alias << " already scheduled, do not enqueue!";
     }
-
-    // wait two seconds until the syncing starts
-    QTimer::singleShot(2000, this, SLOT(slotScheduleFolderSync()));
+    // wait a moment until the syncing starts
+    QTimer::singleShot(500, this, SLOT(slotScheduleFolderSync()));
 }
 
 void FolderMan::setSyncEnabled( bool enabled )
@@ -537,14 +546,17 @@ void FolderMan::removeFolder( const QString& alias )
     if( _folderMap.contains( alias )) {
         qDebug() << "Removing " << alias;
         f = _folderMap.take( alias );
-        if(f) {
-            f->wipe();
-        }
     } else {
         qDebug() << "!! Can not remove " << alias << ", not in folderMap.";
     }
 
     if( f ) {
+        f->wipe();
+
+        // can be removed if we are able to delete the folder object.
+        f->setSyncEnabled(false);
+
+        // remove the folder configuration
         QFile file( _folderConfigPath + QLatin1Char('/') + f->configFile() );
         if( file.exists() ) {
             qDebug() << "Remove folder config file " << file.fileName();
