@@ -34,7 +34,6 @@
 
 #include "creds/credentialsfactory.h"
 #include "creds/abstractcredentials.h"
-#include "creds/shibbolethcredentials.h"
 
 Q_DECLARE_METATYPE(QTimer*)
 
@@ -42,12 +41,15 @@ namespace Mirall {
 
 AbstractNetworkJob::AbstractNetworkJob(Account *account, const QString &path, QObject *parent)
     : QObject(parent)
+    , _duration(0)
     , _ignoreCredentialFailure(false)
     , _reply(0)
     , _account(account)
     , _path(path)
-    , _timer(0)
 {
+    _timer.setSingleShot(true);
+    _timer.setInterval(10*1000); // default to 10 seconds.
+    connect(&_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
 }
 
 void AbstractNetworkJob::setReply(QNetworkReply *reply)
@@ -61,21 +63,15 @@ void AbstractNetworkJob::setReply(QNetworkReply *reply)
 void AbstractNetworkJob::setTimeout(qint64 msec)
 {
     qDebug() << Q_FUNC_INFO << msec;
-    if (_timer)
-        _timer->deleteLater();
-    _timer = new QTimer(this);
-    _timer->setSingleShot(true);
-    connect(_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
-    _timer->start(msec);
+
+    _timer.start(msec);
 }
 
 void AbstractNetworkJob::resetTimeout()
 {
-    if( _timer ) {
-        qint64 interval = _timer->interval();
-        _timer->stop();
-        _timer->start(interval);
-    }
+    qint64 interval = _timer.interval();
+    _timer.stop();
+    _timer.start(interval);
 }
 
 void AbstractNetworkJob::setIgnoreCredentialFailure(bool ignore)
@@ -100,7 +96,7 @@ void AbstractNetworkJob::setupConnections(QNetworkReply *reply)
 
 QNetworkReply* AbstractNetworkJob::addTimer(QNetworkReply *reply)
 {
-    reply->setProperty("timer", QVariant::fromValue(_timer));
+    reply->setProperty("timer", QVariant::fromValue(&_timer));
     return reply;
 }
 
@@ -137,11 +133,21 @@ QNetworkReply *AbstractNetworkJob::headRequest(const QUrl &url)
 
 void AbstractNetworkJob::slotFinished()
 {
+    _timer.stop();
+
     if( _reply->error() != QNetworkReply::NoError ) {
         qDebug() << Q_FUNC_INFO << _reply->error() << _reply->errorString();
+        if (_reply->error() == QNetworkReply::ProxyAuthenticationRequiredError) {
+            qDebug() << Q_FUNC_INFO << _reply->rawHeader("Proxy-Authenticate");
+        }
         emit networkError(_reply);
     }
-    finished();
+
+    // get the Date timestamp from reply
+    _responseTimestamp = QString::fromAscii(_reply->rawHeader("Date"));
+    _duration = _durationTimer.elapsed();
+
+    bool discard = finished();
     AbstractCredentials *creds = _account->credentials();
     if (!creds->stillValid(_reply) &&! _ignoreCredentialFailure
             && _account->state() != Account::InvalidCredidential) {
@@ -151,9 +157,25 @@ void AbstractNetworkJob::slotFinished()
         // but try to re-sign in.
         connect( creds, SIGNAL(fetched()),
                  qApp, SLOT(slotCredentialsFetched()), Qt::UniqueConnection);
-        creds->invalidateAndFetch(_account);   // this triggers Application::runValidator when the credidentials are fetched
+        if (creds->ready()) {
+            creds->invalidateAndFetch(_account);
+        } else {
+            creds->fetch(_account);
+        }
     }
-    deleteLater();
+    if (discard) {
+        deleteLater();
+    }
+}
+
+quint64 AbstractNetworkJob::duration()
+{
+    return _duration;
+}
+
+QString AbstractNetworkJob::responseTimestamp()
+{
+    return _responseTimestamp;
 }
 
 AbstractNetworkJob::~AbstractNetworkJob() {
@@ -162,6 +184,10 @@ AbstractNetworkJob::~AbstractNetworkJob() {
 
 void AbstractNetworkJob::start()
 {
+    _timer.start();
+    _durationTimer.start();
+    _duration = 0;
+
     qDebug() << "!!!" << metaObject()->className() << "created for" << account()->url() << "querying" << path();
 }
 
@@ -203,7 +229,7 @@ void RequestEtagJob::start()
     AbstractNetworkJob::start();
 }
 
-void RequestEtagJob::finished()
+bool RequestEtagJob::finished()
 {
     if (reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 207) {
         // Parse DAV response
@@ -222,6 +248,7 @@ void RequestEtagJob::finished()
         }
         emit etagRetreived(etag);
     }
+    return true;
 }
 
 /*********************************************************************************************/
@@ -240,9 +267,10 @@ void MkColJob::start()
    AbstractNetworkJob::start();
 }
 
-void MkColJob::finished()
+bool MkColJob::finished()
 {
     emit finished(reply()->error());
+    return true;
 }
 
 /*********************************************************************************************/
@@ -272,7 +300,7 @@ void LsColJob::start()
     AbstractNetworkJob::start();
 }
 
-void LsColJob::finished()
+bool LsColJob::finished()
 {
     if (reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 207) {
         // Parse DAV response
@@ -298,15 +326,23 @@ void LsColJob::finished()
         }
         emit directoryListing(folders);
     }
+    return true;
 }
 
 /*********************************************************************************************/
 
+namespace {
+const char statusphpC[] = "status.php";
+const char owncloudDirC[] = "owncloud/";
+}
+
 CheckServerJob::CheckServerJob(Account *account, bool followRedirect, QObject *parent)
-    : AbstractNetworkJob(account, QLatin1String("status.php") , parent)
+    : AbstractNetworkJob(account, QLatin1String(statusphpC) , parent)
     , _followRedirects(followRedirect)
+    , _subdirFallback(false)
     , _redirectCount(0)
 {
+    setIgnoreCredentialFailure(true);
 }
 
 void CheckServerJob::start()
@@ -338,7 +374,7 @@ bool CheckServerJob::installed(const QVariantMap &info)
     return info.value(QLatin1String("installed")).toBool();
 }
 
-void CheckServerJob::finished()
+bool CheckServerJob::finished()
 {
     account()->setSslConfiguration(reply()->sslConfiguration());
 
@@ -357,10 +393,20 @@ void CheckServerJob::finished()
             resetTimeout();
             setReply(getRequest(redirectUrl));
             setupConnections(reply());
-            return;
+            return false;
         }
     }
 
+    // The serverInstalls to /owncloud. Let's try that if the file wasn't found
+    // at the original location
+    if ((reply()->error() == QNetworkReply::ContentNotFoundError) && (!_subdirFallback)) {
+        _subdirFallback = true;
+        setPath(QLatin1String(owncloudDirC)+QLatin1String(statusphpC));
+        start();
+        qDebug() << "Retrying with" << reply()->url();
+        return false;
+    }
+
     bool success = false;
     QVariantMap status = QtJson::parse(QString::fromUtf8(reply()->readAll()), success).toMap();
     // empty or invalid response
@@ -375,7 +421,9 @@ void CheckServerJob::finished()
         emit instanceFound(reply()->url(), status);
     } else {
         qDebug() << "No proper answer on " << requestedUrl;
+        emit instanceNotFound(reply());
     }
+    return true;
 }
 
 /*********************************************************************************************/
@@ -425,7 +473,7 @@ QList<QByteArray> PropfindJob::properties() const
     return _properties;
 }
 
-void PropfindJob::finished()
+bool PropfindJob::finished()
 {
     int http_result_code = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
@@ -460,6 +508,7 @@ void PropfindJob::finished()
         qDebug() << "Quota request *not* successful, http result code is" << http_result_code
                  << (http_result_code == 302 ? reply()->header(QNetworkRequest::LocationHeader).toString()  : QLatin1String(""));
     }
+    return true;
 }
 
 /*********************************************************************************************/
@@ -476,9 +525,10 @@ void EntityExistsJob::start()
     AbstractNetworkJob::start();
 }
 
-void EntityExistsJob::finished()
+bool EntityExistsJob::finished()
 {
     emit exists(reply());
+    return true;
 }
 
 /*********************************************************************************************/
@@ -509,7 +559,7 @@ void CheckQuotaJob::start()
     AbstractNetworkJob::start();
 }
 
-void CheckQuotaJob::finished()
+bool CheckQuotaJob::finished()
 {
     if (reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 207) {
         // Parse DAV response
@@ -523,15 +573,18 @@ void CheckQuotaJob::finished()
                     reader.namespaceUri() == QLatin1String("DAV:")) {
                 QString name = reader.name().toString();
                 if (name == QLatin1String("quota-available-bytes")) {
-                    quotaAvailableBytes = reader.readElementText().toLongLong();
+                    // I have seen the server returning frational bytes:
+                    //   <d:quota-available-bytes>1374532061.2</d:quota-available-bytes>
+                    quotaAvailableBytes = reader.readElementText().toDouble();
                 } else if (name == QLatin1String("quota-used-bytes")) {
-                    quotaUsedBytes = reader.readElementText().toLongLong();
+                    quotaUsedBytes = reader.readElementText().toDouble();
                 }
             }
         }
         qint64 total = quotaUsedBytes + quotaAvailableBytes;
         emit quotaRetrieved(total, quotaUsedBytes);
     }
+    return true;
 }
 
 NetworkJobTimeoutPauser::NetworkJobTimeoutPauser(QNetworkReply *reply)
