@@ -19,7 +19,11 @@
 #include "mirall/utility.h"
 #include "mirall/progressdispatcher.h"
 #include "mirall/owncloudsetupwizard.h"
-#include "mirall/settingsdialog.h"
+#if defined(Q_OS_MAC)
+#    include "mirall/settingsdialogmac.h"
+#else
+#    include "mirall/settingsdialog.h"
+#endif
 #include "mirall/logger.h"
 #include "mirall/logbrowser.h"
 #include "mirall/account.h"
@@ -34,7 +38,11 @@ namespace Mirall {
 ownCloudGui::ownCloudGui(Application *parent) :
     QObject(parent),
     _tray(0),
-    _settingsDialog(0),
+#if defined(Q_OS_MAC)
+    _settingsDialog(new SettingsDialogMac(this)),
+#else
+    _settingsDialog(new SettingsDialog(this)),
+#endif
     _logBrowser(0),
     _contextMenu(0),
     _recentActionsMenu(0),
@@ -64,8 +72,6 @@ ownCloudGui::ownCloudGui(Application *parent) :
     ProgressDispatcher *pd = ProgressDispatcher::instance();
     connect( pd, SIGNAL(progressInfo(QString,Progress::Info)), this,
              SLOT(slotUpdateProgress(QString,Progress::Info)) );
-    connect( pd, SIGNAL(progressSyncProblem(QString,Progress::SyncProblem)),
-             SLOT(slotProgressSyncProblem(QString,Progress::SyncProblem)));
 
     FolderMan *folderMan = FolderMan::instance();
     connect( folderMan, SIGNAL(folderSyncStateChange(QString)),
@@ -85,7 +91,7 @@ void ownCloudGui::slotOpenSettingsDialog( bool openSettings )
     // if account is set up, start the configuration wizard.
     if( AccountManager::instance()->account() ) {
         if( openSettings ) {
-            if (_settingsDialog.isNull()) {
+            if (_settingsDialog.isNull() || !_settingsDialog->isVisible()) {
                 slotShowSettings();
             } else {
                 _settingsDialog->close();
@@ -140,6 +146,7 @@ void ownCloudGui::slotOpenPath(const QString &path)
 void ownCloudGui::slotAccountStateChanged()
 {
     setupContextMenu();
+    slotComputeOverallSyncStatus();
 }
 
 void ownCloudGui::startupConnected( bool connected, const QStringList& fails )
@@ -154,6 +161,8 @@ void ownCloudGui::startupConnected( bool connected, const QStringList& fails )
     }
 
     _startupFails = fails; // store that for the settings dialog once it appears.
+    if( !_settingsDialog.isNull() )
+        _settingsDialog->setGeneralErrors( _startupFails );
 
 }
 
@@ -165,6 +174,11 @@ void ownCloudGui::slotComputeOverallSyncStatus()
             _tray->setToolTip(tr("Please sign in"));
             return;
         }
+        if (a->state() == Account::Disconnected) {
+            _tray->setIcon(Theme::instance()->syncStateIcon( SyncResult::Unavailable, true));
+            _tray->setToolTip(tr("Disconnected from server"));
+            return;
+        }
     }
     // display the info of the least successful sync (eg. not just display the result of the latest sync
     QString trayMessage;
@@ -286,9 +300,6 @@ void ownCloudGui::setupContextMenu()
         _contextMenu->addAction(_actionLogin);
     }
     _contextMenu->addAction(_actionQuit);
-
-    // Populate once at start
-    slotRebuildRecentMenus();
 }
 
 
@@ -315,10 +326,9 @@ void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QStrin
 void ownCloudGui::slotFolderOpenAction( const QString& alias )
 {
     Folder *f = FolderMan::instance()->folder(alias);
-    qDebug() << "opening local url " << f->path();
     if( f ) {
-        QUrl url(f->path(), QUrl::TolerantMode);
-        url.setScheme( QLatin1String("file") );
+        qDebug() << "opening local url " << f->path();
+        QUrl url = QUrl::fromLocalFile(f->path());
 
 #ifdef Q_OS_WIN
         // work around a bug in QDesktopServices on Win32, see i-net
@@ -372,50 +382,18 @@ void ownCloudGui::slotRefreshQuotaDisplay( qint64 total, qint64 used )
     _actionQuota->setText(tr("%1% of %2 in use").arg(percentFormatted).arg(totalFormatted));
 }
 
-void ownCloudGui::slotProgressSyncProblem(const QString& folder, const Progress::SyncProblem& problem)
-{
-    Q_UNUSED(folder);
-    Q_UNUSED(problem);
-
-    // display a warn icon if warnings happend.
-    QIcon warnIcon(":/mirall/resources/warning-16");
-    _actionRecent->setIcon(warnIcon);
-
-    slotRebuildRecentMenus();
-}
-
 void ownCloudGui::slotRebuildRecentMenus()
 {
     _recentActionsMenu->clear();
-    const QList<Progress::Info>& progressInfoList = ProgressDispatcher::instance()->recentChangedItems(5);
-
-    if( progressInfoList.size() == 0 ) {
-        _recentActionsMenu->addAction(tr("No items synced recently"))->setEnabled(false);
-    } else {
-        QListIterator<Progress::Info> i(progressInfoList);
-
-        while(i.hasNext()) {
-            Progress::Info info = i.next();
-            QString kindStr = Progress::asResultString(info);
-            QString timeStr = info.timestamp.toString("hh:mm");
-
-            QString actionText = tr("%1 (%2, %3)").arg(info.current_file).arg(kindStr).arg(timeStr);
-            QAction *action = _recentActionsMenu->addAction( actionText );
-
-            Folder *folder = FolderMan::instance()->folder(info.folder);
-            if (folder) {
-                QString fullPath = folder->path() + '/' + info.current_file;
-                if (QFile(fullPath).exists()) {
-                    _recentItemsMapper->setMapping(action, fullPath);
-                    connect(action, SIGNAL(triggered()), _recentItemsMapper, SLOT(map()));
-                } else {
-                    action->setEnabled(false);
-                }
-            }
+    if (!_recentItemsActions.isEmpty()) {
+        foreach(QAction *a, _recentItemsActions) {
+            _recentActionsMenu->addAction(a);
         }
+        _recentActionsMenu->addSeparator();
+    } else {
+        _recentActionsMenu->addAction(tr("No items synced recently"))->setEnabled(false);
     }
     // add a more... entry.
-    _recentActionsMenu->addSeparator();
     _recentActionsMenu->addAction(_actionRecent);
 }
 
@@ -424,25 +402,47 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const Progress::Info
 {
     Q_UNUSED(folder);
 
-    // shows an entry in the context menu.
-    QString curAmount = Utility::octetsToString(progress.overall_current_bytes);
-    QString totalAmount = Utility::octetsToString(progress.overall_transmission_size);
-    _actionStatus->setText(tr("Syncing %1 of %2 (%3 of %4) ").arg(progress.current_file_no)
-                           .arg(progress.overall_file_count).arg(curAmount, totalAmount));
+    quint64 completedSize = progress.completedSize();
+    quint64 currentFile =  progress._completedFileCount + progress._currentItems.count();
+    QString s1 = Utility::octetsToString( completedSize );
+    QString s2 = Utility::octetsToString( progress._totalSize );
 
-    // wipe the problem list at start of sync.
-    if( progress.kind == Progress::StartSync ) {
-        _actionRecent->setIcon( QIcon() ); // Fixme: Set a "in-progress"-item eventually.
-    }
+    _actionStatus->setText(tr("Syncing %1 of %2 (%3 of %4)")
+        .arg(currentFile).arg(progress._totalFileCount).arg(s1, s2));
+
+    _actionRecent->setIcon( QIcon() ); // Fixme: Set a "in-progress"-item eventually.
+
+    if (!progress._lastCompletedItem.isEmpty()) {
+
+        if (Progress::isWarningKind(progress._lastCompletedItem._status)) {
+            // display a warn icon if warnings happend.
+            QIcon warnIcon(":/mirall/resources/warning-16");
+            _actionRecent->setIcon(warnIcon);
+        }
+
+        QString kindStr = Progress::asResultString(progress._lastCompletedItem);
+        QString timeStr = QTime::currentTime().toString("hh:mm");
+        QString actionText = tr("%1 (%2, %3)").arg(progress._lastCompletedItem._file, kindStr, timeStr);
+        QAction *action = new QAction(actionText, this);
+        Folder *f = FolderMan::instance()->folder(folder);
+        if (f) {
+            QString fullPath = f->path() + '/' + progress._lastCompletedItem._file;
+            if (QFile(fullPath).exists()) {
+                _recentItemsMapper->setMapping(action, fullPath);
+                connect(action, SIGNAL(triggered()), _recentItemsMapper, SLOT(map()));
+            } else {
+                action->setEnabled(false);
+            }
+        }
+        if (_recentItemsActions.length() > 5) {
+            _recentItemsActions.takeFirst()->deleteLater();
+        }
+        _recentItemsActions.append(action);
 
-    // If there was a change in the file list, redo the progress menu.
-    if( progress.kind == Progress::EndDownload || progress.kind == Progress::EndUpload ||
-            progress.kind == Progress::EndDelete || progress.kind == Progress::EndRename ) {
         slotRebuildRecentMenus();
     }
 
-    if (progress.kind == Progress::EndSync) {
-        slotRebuildRecentMenus();  // show errors.
+    if (progress._completedFileCount == progress._totalFileCount) {
         QTimer::singleShot(2000, this, SLOT(slotDisplayIdle()));
     }
 }
@@ -465,14 +465,17 @@ void ownCloudGui::slotShowGuiMessage(const QString &title, const QString &messag
 void ownCloudGui::slotShowSettings()
 {
     if (_settingsDialog.isNull()) {
-        _settingsDialog = new SettingsDialog(this);
+        _settingsDialog =
+#if defined(Q_OS_MAC)
+                new SettingsDialogMac(this);
+#else
+                new SettingsDialog(this);
+#endif
         _settingsDialog->setAttribute( Qt::WA_DeleteOnClose, true );
         _settingsDialog->show();
     }
-
     _settingsDialog->setGeneralErrors( _startupFails );
     Utility::raiseDialog(_settingsDialog.data());
-    _settingsDialog->slotRefreshResultList();
 }
 
 void ownCloudGui::slotShowSyncProtocol()
